#
# spec file for package xstream
#
# Copyright (c) 2023 SUSE LLC
# Copyright (c) 2000-2007, JPackage Project
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%bcond_with  hibernate
%bcond_with  stax
Name:           xstream
Version:        1.4.20
Release:        4.3
Summary:        Java XML serialization library
License:        BSD-3-Clause
Group:          Development/Libraries/Java
URL:            https://x-stream.github.io/
Source0:        https://repo1.maven.org/maven2/com/thoughtworks/%{name}/%{name}-distribution/%{version}/%{name}-distribution-%{version}-src.zip
Patch0:         Revert-MXParser-changes.patch
BuildRequires:  fdupes
BuildRequires:  java-devel >= 1.8
BuildRequires:  maven-local
BuildRequires:  unzip
BuildRequires:  mvn(cglib:cglib)
BuildRequires:  mvn(dom4j:dom4j)
BuildRequires:  mvn(javax.activation:activation)
BuildRequires:  mvn(javax.xml.bind:jaxb-api)
BuildRequires:  mvn(joda-time:joda-time)
BuildRequires:  mvn(net.sf.kxml:kxml2-min)
BuildRequires:  mvn(org.apache.felix:maven-bundle-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-antrun-plugin)
BuildRequires:  mvn(org.apache.maven.plugins:maven-enforcer-plugin)
BuildRequires:  mvn(org.codehaus.jettison:jettison)
BuildRequires:  mvn(org.codehaus.mojo:build-helper-maven-plugin)
BuildRequires:  mvn(org.codehaus.woodstox:woodstox-core-asl)
BuildRequires:  mvn(org.jdom:jdom)
BuildRequires:  mvn(org.jdom:jdom2)
BuildRequires:  mvn(xom:xom)
BuildRequires:  mvn(xpp3:xpp3)
BuildRequires:  mvn(xpp3:xpp3_min)
BuildArch:      noarch
%if %{with hibernate}
BuildRequires:  mvn(org.hibernate:hibernate-core)
BuildRequires:  mvn(org.hibernate:hibernate-envers)
%endif
%if %{with stax}
BuildRequires:  mvn(stax:stax)
BuildRequires:  mvn(stax:stax-api)
%endif

%description
XStream is a simple library to serialize objects to XML
and back again. A high level facade is supplied that
simplifies common use cases. Custom objects can be serialized
without need for specifying mappings. Speed and low memory
footprint are a crucial part of the design, making it suitable
for large object graphs or systems with high message throughput.
No information is duplicated that can be obtained via reflection.
This results in XML that is easier to read for humans and more
compact than native Java serialization. XStream serializes internal
fields, including private and final. Supports non-public and inner
classes. Classes are not required to have default constructor.
Duplicate references encountered in the object-model will be
maintained. Supports circular references. By implementing an
interface, XStream can serialize directly to/from any tree
structure (not just XML). Strategies can be registered allowing
customization of how particular types are represented as XML.
When an exception occurs due to malformed XML, detailed diagnostics
are provided to help isolate and fix the problem.

%package        javadoc
Summary:        Javadoc for %{name}
Group:          Documentation/HTML

%description    javadoc
%{name} API documentation.

%if %{with hibernate}
%package        hibernate
Summary:        The hibernate module for %{name}
Group:          Development/Libraries/Java
Requires:       %{name} = %{version}-%{release}

%description    hibernate
This package contains the hibernate module for %{name}.
%endif

%package        benchmark
Summary:        The benchmark module for %{name}
Group:          Development/Libraries/Java
Requires:       %{name} = %{version}-%{release}

%description    benchmark
This package contains the benchmark module for %{name}.

%package parent
Summary:        Parent POM for %{name}
Group:          Development/Libraries/Java
Requires:       %{name} = %{version}-%{release}

%description parent
Parent POM for %{name}.

%prep
%setup -q
%patch0 -p1
find . -name "*.class" -print -delete
find . -name "*.jar" -print -delete

# Require org.codehaus.xsite:xsite-maven-plugin
%pom_disable_module xstream-distribution

# missing artifacts:
#  org.openjdk.jmh:jmh-core:jar:1.11.1
#  org.openjdk.jmh:jmh-generator-annprocess:jar:1.11.1
%pom_disable_module xstream-jmh

%pom_remove_plugin :xsite-maven-plugin
# Unwanted
%pom_remove_plugin :maven-source-plugin
%pom_remove_plugin :maven-dependency-plugin
%pom_remove_plugin :maven-eclipse-plugin
%pom_remove_plugin :maven-release-plugin

%pom_xpath_set "pom:dependency[pom:groupId = 'org.codehaus.woodstox' ]/pom:artifactId" woodstox-core-asl
%pom_xpath_set "pom:dependency[pom:groupId = 'org.codehaus.woodstox' ]/pom:artifactId" woodstox-core-asl xstream
%pom_xpath_set "pom:dependency[pom:groupId = 'cglib' ]/pom:artifactId" cglib
%pom_xpath_set "pom:dependency[pom:groupId = 'cglib' ]/pom:artifactId" cglib xstream
# Require unavailable proxytoys:proxytoys
%pom_remove_plugin :maven-dependency-plugin xstream

%pom_xpath_set "pom:project/pom:dependencies/pom:dependency[pom:groupId = 'cglib' ]/pom:artifactId" cglib xstream-hibernate
%pom_xpath_inject "pom:project/pom:dependencies/pom:dependency[pom:groupId = 'junit' ]" "<scope>test</scope>" xstream-hibernate
%pom_remove_plugin :maven-dependency-plugin xstream-hibernate

%pom_xpath_inject "pom:project/pom:dependencies/pom:dependency[pom:groupId = 'junit' ]" "<scope>test</scope>" xstream-benchmark

%pom_xpath_remove "pom:project/pom:profiles/pom:profile[pom:id = 'jdk12-ge' ]"

%pom_xpath_inject "pom:project/pom:profiles" "
    <profile>
      <id>jdk18-ge</id>
      <activation>
        <jdk>[18,)</jdk>
      </activation>
      <properties>
        <version.java.5>1.8</version.java.5>
        <version.java.6>1.8</version.java.6>
        <version.java.source>1.8</version.java.source>
        <version.java.target>1.8</version.java.target>
      </properties>
    </profile>
    <profile>
      <id>jdk12-ge-18</id>
      <activation>
        <jdk>[12,18)</jdk>
      </activation>
      <properties>
        <version.java.5>1.7</version.java.5>
        <version.java.6>1.7</version.java.6>
        <version.java.source>1.7</version.java.source>
        <version.java.target>1.7</version.java.target>
      </properties>
    </profile>"

%if %{without hibernate}
%pom_disable_module xstream-hibernate
%endif

%if %{without stax}
%pom_remove_dep -r stax:
rm -f xstream/src/java/com/thoughtworks/xstream/io/xml/BEAStaxDriver.java \
    xstream-benchmark/src/java/com/thoughtworks/xstream/tools/benchmark/products/XStreamBEAStax.java
%endif

%{mvn_file} :%{name} %{name}/%{name} %{name}
%{mvn_file} :%{name}-benchmark %{name}/%{name}-benchmark %{name}-benchmark

%{mvn_package} :%{name}

%build
%{mvn_build} -f -s -- \
    -Dproject.build.outputTimestamp=$(date -u -d @${SOURCE_DATE_EPOCH:-$(date +%%s)} +%%Y-%%m-%%dT%%H:%%M:%%SZ) \
	-Dversion.java.source=8 -Dversion.java.target=8

%install
%mvn_install
%fdupes %{buildroot}%{_javadocdir}

%files -f .mfiles
%license LICENSE.txt
%doc README.txt
%dir %{_javadir}/%{name}

%files parent -f .mfiles-%{name}-parent
%if %{with hibernate}
%files hibernate -f .mfiles-%{name}-hibernate
%endif

%files benchmark -f .mfiles-%{name}-benchmark

%files javadoc -f .mfiles-javadoc
%license LICENSE.txt

%changelog
* Wed Oct 25 2023 Fridrich Strba <fstrba@suse.com>
- Build with source/target 8 with java 18+
* Sat Sep  9 2023 Fridrich Strba <fstrba@suse.com>
- Reproducible builds: use SOURCE_DATE_EPOCH for timestamp
* Fri Aug 25 2023 Fridrich Strba <fstrba@suse.com>
- Make dependency on bea-stax optional and disable it by default
* Fri Mar 17 2023 Fridrich Strba <fstrba@suse.com>
- Upgrade to 1.4.20
  * Security fixes
    + This maintenance release addresses the security
    vulnerabilities CVE-2022-40151 (bsc#1203520) and
    CVE-2022-41966 (bsc#1206729), causing a Denial of Service by
    raising a stack overflow. It also provides new converters for
    Optional and Atomic types.
  * Major changes
    + #308: Add converter for AtomicBoolean, AtomicInteger,
    AtomicLong, and AtomicReference of package
    java.util.concurrent.atomic.
    + #293: Add converter for Optional, OptionalDouble, OptionalInt,
    and OptionalLong of package java.util.
  * Minor changes
    + #287: Close stream opened from provided URL.
    + #284: Fix disabling check against hash code attack with
    XStream.setCollectionUpdateLimit(0).
  * Stream compatibility
    + The atomic types with new converters of package
    java.util.concurrent.atomic, that have been written with
    previous versions of XStream, can still be deserialized.
    + The Optional types with new converters of package java.util,
    that have been written with previous versions of XStream,
    can still be deserialized.
    + The WildcardTypePermission allows by default no longer
    anonymous class types.
  * API changes
    + Added c.t.x.converters.extended.AtomicBooleanConverter.
    + Added c.t.x.converters.extended.AtomicIntegerConverter.
    + Added c.t.x.converters.extended.AtomicLongConverter.
    + Added c.t.x.converters.extended.AtomicReferenceConverter.
    + Added c.t.x.converters.extended.OptionalConverter.
    + Added c.t.x.converters.extended.OptionalDoubleConverter.
    + Added c.t.x.converters.extended.OptionalIntConverter.
    + Added c.t.x.converters.extended.OptionalLongConverter.
    + Added c.t.x.security.WildcardTypePermission
    .WildcardTypePermission(boolean,String[]).
* Wed Mar 30 2022 Fridrich Strba <fstrba@suse.com>
- Build against the standalone JavaEE modules unconditionally
* Fri Mar 18 2022 Fridrich Strba <fstrba@suse.com>
- Build against standalone activation-api and jaxb-api on systems
  where the JavaEE modules are not part of JDK
* Fri Feb  4 2022 Fridrich Strba <fstrba@suse.com>
- Upgrade to 1.4.19
  * Security fixes
    + This maintenance release addresses the security vulnerability
    CVE-2021-43859, bsc#1195458, when unmarshalling highly
    recursive collections or maps causing a Denial of Service.
  * API changes
    + Added c.t.x.XStream.COLLECTION_UPDATE_LIMIT and
    c.t.x.XStream.COLLECTION_UPDATE_SECONDS.
    + Added c.t.x.XStream.setCollectionUpdateLimit(int).
    + Added c.t.x.core.SecurityUtils.
    + Added c.t.x.security.AbstractSecurityException and
    c.t.x.security.InputManipulationException.
    + c.t.x.security.InputManipulationException derives now from
    c.t.x.security.AbstractSecurityException.
* Tue Sep 28 2021 Fridrich Strba <fstrba@suse.com>
- Upgrade to 1.4.18
  * Security fixes
    + This maintenance release addresses following security
    vulnerabilities, when unmarshalling with an XStream instance
    using the default blacklist of an uninitialized security
    framework. XStream is therefore now using a whitelist by
    default. (CVE-2021-39139, CVE-2021-39140, CVE-2021-39141,
    CVE-2021-39144, CVE-2021-39145, CVE-2021-39146,
    CVE-2021-39147, CVE-2021-39148, CVE-2021-39149,
    CVE-2021-39150, CVE-2021-39151, CVE-2021-39152,
    CVE-2021-39153, CVE-2021-39154, bsc#1189798)
  * Minor changes
    + Support serializable types with non-serializable parent with
    PureJavaReflectionConverter.
  * Stream compatibility
    + Starting with version 1.14.12 nine years ago, XStream contains
    a Security Framework to implement a black- or whitelist for
    the allowed types at deserialization time. Until version
    1.4.17, XStream kept a default blacklist in order to deny all
    types of the Java runtime, which are used for all kinds of
    security attacks, in order to guarantee optimal runtime
    compatibility for existing users. However, this approach has
    failed. The last months have shown, that the Java runtime
    alone contains dozens of types that can be used for an attack,
    not even looking at the 3rd party libraries on a classpath.
    The new version of XStream uses therefore now by default a
    whitelist, which is recommended since nine years. It also has
    been complaining on the console for a long time about an
    uninitialized security framework the first time it was run.
    Anyone who has followed the advice and initialized the
    security framework for their own scenario can easily update
    to the new version without any problem. Everyone else will
    have to do a proper initialization now, otherwise the new
    version will fail with certainty at deserialization time.
- Modified patch:
  * Revert-MXParser-changes.patch
    + rediff to changed context
* Mon May 31 2021 Fridrich Strba <fstrba@suse.com>
- Upgrade to 1.4.17
  * Security fix:
  * bsc#1186651, CVE-2021-29505: potential code execution when
    unmarshalling with XStream instances using an uninitialized
    security framework
* Thu Apr 15 2021 Fridrich Strba <fstrba@suse.com>
- Upgrade to 1.4.16
  * Security fixes:
    + bsc#1184796, CVE-2021-21351: remote attacker to load and
    execute arbitrary code
    + bsc#1184797, CVE-2021-21349: SSRF can lead to a remote
    attacker to request data from internal resources
    + bsc#1184380, CVE-2021-21350: arbitrary code execution
    + bsc#1184374, CVE-2021-21348: remote attacker could cause
    denial of service by consuming maximum CPU time
    + bsc#1184378, CVE-2021-21347: remote attacker to load and
    execute arbitrary code from a remote host
    + bsc#1184375, CVE-2021-21344: remote attacker could load and
    execute arbitrary code from a remote host
    + bsc#1184379, CVE-2021-21342: server-side forgery
    + bsc#1184377, CVE-2021-21341: remote attacker could cause a
    denial of service by allocating 100%% CPU time
    + bsc#1184373, CVE-2021-21346: remote attacker could load and
    execute arbitrary code
    + bsc#1184372, CVE-2021-21345: remote attacker with sufficient
    rights could execute commands
    + bsc#1184376, CVE-2021-21343: replace or inject objects, that
    result in the deletion of files on the local host
- Add patch:
  * Revert-MXParser-changes.patch
    + revert changes that would force us to add new dependency
* Tue Mar  9 2021 Johannes Renner <jrenner@suse.com>
- Upgrade to 1.4.15
  * fixes bsc#1180146, CVE-2020-26258 and bsc#1180145,
    CVE-2020-26259
- Upgrade to 1.4.14
  * fixes bsc#1180994, CVE-2020-26217
- Update xstream to 1.4.15~susemanager
  Removed:
  * xstream_1_4_10-jdk11.patch
  * xstream_1_4_10-buildsh-sle12.patch
  * build.sh
* Tue Mar  5 2019 Frantisek Kobzik <fkobzik@suse.com>
- Update xstream to 1.4.10
  Added:
  * xstream_1_4_10-jdk11.patch
  * xstream_1_4_10-buildsh-sle12.patch
  * xstream-XSTREAM_1_4_10.tar.gz
  Removed:
  * 0001-Prevent-deserialization-of-void.patch
  * xstream-XSTREAM_1_4_9.tar.gz
  * xstream-XSTREAM_1_4_9-jdk11.patch
- Major changes:
- New XStream artifact with -java7 appended as version suffix for a library explicitly without the Java 8 stuff (lambda expression support, converters for java.time.* package).
- Fix PrimitiveTypePermission to reject type void to prevent CVE-2017-7957 with an initialized security framework.
- Improve performance by minimizing call stack of mapper chain.
- XSTR-774: Add converters for types of java.time, java.time.chrono, and java.time.temporal packages (converters for LocalDate, LocalDateTime, LocalTime, OffsetDateTime, and ZonedDateTime by Matej Cimbora).
- JavaBeanConverter does not respect ignored unknown elements.
- Add XStream.setupDefaultSecurity to initialize security framework with defaults of XStream 1.5.x.
- Emit error warning if security framework has not been initialized and the XStream instance is vulnerable to known exploits.
* Tue Feb  5 2019 michele.bologna@suse.com
- Feat: modify patch to be compatible with JDK 11 building
  Added:
  * xstream-XSTREAM_1_4_9-jdk11.patch
  Removed:
  * xstream-XSTREAM_1_4_9-jdk9.patch
* Tue Dec 11 2018 moio@suse.com
- fixes for SLE 15 compatibility
* Fri Dec  1 2017 mc@suse.com
- fix possible Denial of Service when unmarshalling void.
  (CVE-2017-7957, bsc#1070731)
  Added:
  * 0001-Prevent-deserialization-of-void.patch
* Tue Nov  7 2017 jgonzalez@suse.com
- Fix build for JDK9
- Disable javadoc generation (broken for SLE15 and Tumbleweed)
- Add:
  * xstream-XSTREAM_1_4_9-jdk9.patch
- Changed:
  * build.sh
* Tue Apr  5 2016 moio@suse.com
- Require building on Java 8, otherwise the LambdaMapper class is skipped
  (issue 30)
* Tue Mar 29 2016 moio@suse.com
- Upgrade to version 1.4.9, which fixes CVE-2016-3674 (bsc#972950)
* Tue Nov 10 2015 moio@suse.com
- Initial version
